"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var CodeMirror = require("codemirror");
require("../style/index.css");
const ESL_MIME_TYPE = "text/x-elephant";
const ESL_EXTENSION = ".esl";
const ESL_NAME = "Elephant";
// Token regexes
var keyword_control_define = /\b(define)\b/m;
var define_class = /[\t ]+[\w-]+/m;
var define_nametag = /[\t ]+[\w-]+/m;
var keyword_control_end = /\bend\b/m;
var continue_newline = /\s*(\.{3})/m;
var support_function_an = /\b(an?)\b/m;
var support_function_auxiliary_verb = /\b(shall|must|should|could|will)\b/m;
var keyword_control_environments = /\b(world|components?|variables?|parameters?|comments?|needs?|relations?|arguments?|subclauses?|(goal|transformation|design)-(requirement|constraint))\b/m;
var support_function_isbe = /\b(is|be)\b/m;
var entity_name_tag_label = /\b[\w-]+:/m;
var support_function_logical = /\b((at[ \t]+most)|(at[ \t]least)|(smaller[ \t]than)|(greater[ \t]than)|(equal[ \t]+to)|not|or|and)\b/m;
var constant_language_types = /\b(real|string)\b/m;
var keyword_control_with = /\bwith[\t ]+(subclauses?|arguments?)\b/m;
var any_token = /\b[\w-]+\b/m;
var property = /\b(property)\b/m;
var bundle_of = /\b(bundle[ \t]+of)\b/m;
// Default state
function default_state() {
    return {
        define: 0,
    };
}
// ESL mode definition
function esl_mode() {
    return {
        startState: default_state,
        token: function (stream, state) {
            // As soon as we find a comment sign, rest of the line is a comment.
            if (stream.peek() === "#") {
                stream.skipToEnd();
                return "comment";
            }
            // Define block "define"
            if (state.define === 0) {
                if (stream.match(keyword_control_define)) {
                    if (!stream.eol() || continue_newline.test(stream)) {
                        state.define = 1;
                    }
                    else {
                        state.define = 0;
                    }
                    return "keyword";
                }
            }
            // Define block "class" (component, variable, etc.)
            else if (state.define === 1) {
                if (stream.match(define_class)) {
                    if (!stream.eol() || continue_newline.test(stream)) {
                        state.define = 2;
                    }
                    else {
                        state.define = 0;
                    }
                    return "type";
                }
                else {
                    state.define = 0;
                }
            }
            // Define block component nametag
            else if (state.define === 2) {
                state.define = 0;
                if (stream.match(define_nametag)) {
                    return "tag";
                }
            }
            // Simple regexes
            if (stream.match(keyword_control_end)) {
                return "keyword";
            }
            else if (stream.match(support_function_an)) {
                return "keyword";
            }
            else if (stream.match(support_function_auxiliary_verb)) {
                return "keyword";
            }
            else if (stream.match(keyword_control_environments)) {
                return "keyword";
            }
            else if (stream.match(support_function_isbe)) {
                return "keyword";
            }
            else if (stream.match(entity_name_tag_label)) {
                return "tag";
            }
            else if (stream.match(support_function_logical)) {
                return "meta";
            }
            else if (stream.match(constant_language_types)) {
                return "meta";
            }
            else if (stream.match(keyword_control_with)) {
                return "keyword";
            }
            else if (stream.match(property)) {
                return "attribute";
            }
            else if (stream.match(bundle_of)) {
                return "keyword";
            }
            // Match any word, but don't style it.
            else if (stream.match(any_token)) {
                state = default_state();
                return null;
            }
            // Nothing matched, reset state and eat a char for breakfast.
            state = default_state();
            stream.next();
            return null;
        }
    };
}
const extension = {
    id: "esl-jupyterlab-codemirror",
    autoStart: true,
    activate: (app) => {
        console.log("JupyterLab extension esl-jupyterlab-codemirror is being activated...");
        CodeMirror.defineMode(ESL_NAME.toLowerCase(), esl_mode);
        CodeMirror.defineMIME(ESL_MIME_TYPE, { name: ESL_NAME.toLowerCase() });
        CodeMirror.modeInfo.push({
            ext: [ESL_EXTENSION.substring(1)],
            mime: ESL_MIME_TYPE,
            mode: ESL_NAME.toLowerCase(),
            name: ESL_NAME
        });
        console.log("Associating .esl format with newly added mode...");
        app.docRegistry.addFileType({
            name: ESL_NAME,
            displayName: "Elephant Specification language File",
            extensions: [ESL_EXTENSION],
            mimeTypes: [ESL_MIME_TYPE],
            iconClass: "jp-MaterialIcon jp-PasteIcon"
        });
        console.log("Done!");
    }
};
exports.default = extension;
